module merten.cgi.cookie;

private import string, c.stdlib;
private import merten.cgi.cgierror;

/* gets and sets cookies. use it directly or via the Request or Response classes */
class Cookie
{
    char[][] fetchedCookies;
    char[][] cookiesToSend;

    /* fetches cookies */
    this()
    {
        fetchedCookies = split(string.toString(getenv((char*)"HTTP_COOKIE")), ";");
    }

    // debug
    this(char[] cookieString)
    {
        fetchedCookies = split(cookieString, ";");
    }

    /* returns the value of cookie. throws CGIError on failure. */
    char[] getCookie(char[] name)
    {
        char[][] cookies = fetchedCookies;
        char[][char[]] result;
        char[][] splitted;
        for(int i; i < cookies.length; i++)
        {
            splitted = split(cookies[i], "=");
            result[splitted[0]] = splitted[1];
        }

        try
        {
            return result[name];
        }
        catch
        {
            throw new CGIError("Couldn't find cooke: " ~ name);
        }
    }

    /* writes new cookie to headers */
    void setCookie(char[] name, char[] value, char[] expires, char[] path, char[] domain, bit secure)
    {
        char[] result;

        if(name.length == 0)
        {
            throw new CGIError("Cookies must have names!");
        }
        else
        {
            result ~= "Set-Cookie: "~name~"="~value;
        }
        if(expires.length > 0)
        {
            result ~= "; expires="~expires;
        }
        if(path.length > 0)
        {
            result ~= "; path="~path;
        }
        if(domain.length > 0)
        {
            result ~= "; domain="~domain;
        }
        if(secure)
        {
            result ~= "; secure";
        }


        cookiesToSend ~= result;
    }

    /* returns an associative array of strings with all cookies found */
    char[][char[]] getAllCookies()
    {
        char[][char[]] result;
        char[][] s;

        for(int i = 0; i < fetchedCookies.length; i++)
        {
            s = split(fetchedCookies[i], "=");
            result[s[0]] = s[1];
        }

        return result;
    }


    /* return properly formatted headers for printing */
    char[] getCookieHeaders()
    {
        return join(cookiesToSend, "\n");
    }
}