module merten.cgi.form;

private import c.stdlib, string, stream;
private import merten.cgi.url;
private import merten.cgi.cgierror;

private char[] getEnv(char[] s)
{
    return string.toString(c.stdlib.getenv((char*)s));
}

/* parses form or querystring data */
private class Parser
{
    char[][char[]] fields;

    /* parses form or url data to an associative array of strings */
    void parse(char[] vars)
    {
        char[][] f1, f2;
        vars = string.replace(vars, "+", "%20");
        f1 = string.split(vars, "&");

        for(int i = 0; i < f1.length; i++)
        {
            f2 = string.split(f1[i], "=");
            if(fields[f2[0]].length > 0) // key aldready exists
            {
                char[] key = fields[f2[0]];
                key ~= ";" ~ f2[1];

                fields[f2[0]] = key;
            }
            else
            {
                try
                {
                    fields[f2[0]] = f2[1];
                }
                catch
                {
                    fields[f2[0]] = "";
                }
            }

        }
    }

    /* fetches the value of a form field. returns empty string on failure.
     * if more than one field has the same name it returns them all, separated
     * by a semicolon.
     */
    char[] getString(char[] key)
    {
        try
        {
            return urldecode(fields[key]);
        }
        catch
        {
            throw new CGIError("Field doesn't exist: " ~ key);
        }
    }

    /* fetches value of a form field as int. returns 0 on failure
     * if more than one field has the same name it returns them all, separated
     * by a semicolon.
     */
    int getInt(char[] key)
    {
        try
        {
            return atoi(urldecode(fields[key]));
        }
        catch
        {
            throw new CGIError(key ~ " is not an int!");
        }
    }

    /* returns formfield values with same names as an array of strings.
     * good for multiple select boxes. */
    char[][] getArray(char[] key)
    {
        char[][] dict, result;
        dict = split(fields[key], ";");

        for(int i; i < dict.length; i++)
        {
            result[i] = urldecode(dict[i]);
        }

        return result;
    }

    unittest
    {
        Parser p = new Parser();
        p.parse("n=M%a5rten+Ask&g=34&hej=hello&hej=hello2");
        assert(p.getString("n") == "Mrten Ask");
        assert(p.getInt("g") == 34);
        assert(p.getArray("hej")[0] == "hello");
        assert(p.getArray("hej")[1] == "hello2");
    }
}


//private import c.stdio;

/* Form parser. reads from stdin. use it directly or via the Request or Response classes */
class Form : Parser
{
    this()
    {
        int contentLength = atoi(getEnv("CONTENT_LENGTH"));
        if(string.icmp(getEnv("REQUEST_METHOD"), "POST") == 0 && (contentLength > 0))
        {
            ubyte[] buffer;
            buffer.length = contentLength;
            stream.stdin.read(buffer);
            parse((char[])buffer);
        }
    }
}



/* querystring parser. use it directly or via the Request or Response classes */
class Querystring : Parser
{
    this()
    {
        char[] qs = getEnv("QUERY_STRING");
        if(qs.length > 0)
        {
            parse(qs);
        }
    }
}