module merten.cgi.main;

private import string, c.stdlib;
private import merten.cgi.cgierror;
private import merten.cgi.form;
private import merten.cgi.cookie;


/**
 * Similar to the Request class in ASP.
 */
class Request
{
    Form f;
    Querystring q;
    Cookie c;
    char[] self;

    this()
    {
        this.f = new Form();
        this.q = new Querystring();
        this.c = new Cookie();

        this.self = string.toString(getenv((char*)"SCRIPT_NAME"));
    }

    /* returns the environment variable */
    char[] serverVariable(char[] name)
    {
        return string.toString(getenv((char*)name));
    }

    /* returns the value of formfield as a string */
    char[] formString(char[] fieldName)
    {
        return f.getString(fieldName);
    }

    /* returns a formfield value as an int */
    int formInt(char[] fieldName)
    {
        return f.getInt(fieldName);
    }

    /* returns all formfields named 'fieldname' as array of strings */
    char[][] formArray(char[] fieldName)
    {
        return f.getArray(fieldName);
    }

    /* returns querystring value as string */
    char[] queryString(char[] name)
    {
        return q.getString(name);
    }

    /* returns querystring value as int */
    int queryInt(char[] name)
    {
        return q.getInt(name);
    }

    /* returns all querystring values with the name 'name' as array of strings */
    char[][] queryArray(char[] name)
    {
        return q.getArray(name);
    }

    /* returns value of cookie 'name' */
    char[] cookie(char[] name)
    {
        return c.getCookie(name);
    }

    /* returns all cookies found as associative array of strings */
    char[][char[]] allCookies()
    {
        return c.getAllCookies();
    }
}

/**
 * Similar to the Response class in ASP.
 */
class Response
{
    Cookie c;
    char[][] headers;
    char[] buffer;

    this()
    {
        c = new Cookie();
    }

    /* sets new cookie. only 'name' is needed. */
    void cookie(char[] name, char[] value, char[] expires, char[] path, char[] domain, bit secure)
    {
        c.setCookie(name, value, expires, path, domain, secure);
    }

    /* writes 'str' to current buffer */
    void write(char[] str)
    {
        buffer ~= str;
    }

    /* writes a new header */
    void header(char[] name, char[] value)
    {
        headers ~= name~": "~value;
    }

    /* redirects the browser to 'location' */
    void redirect(char[] location)
    {
        location = "Location: "~location;
        headers ~= location;
    }

    /* returns properly formatted headers for printing. if
     * Content-type isn't in there, text/html is assumed.
     * don't forget to add two linebreaks before any content!
     */
    char[] getHeaders()
    {
        bit allSet = 0;
        for(int i; i < headers.length; i++)
        {
            if(string.icmp(headers[i][0..12], "Content-type") == 0)
            {
                allSet = 1;
                break;
            }
        }
        if(!allSet)
        {
            headers ~= "Content-type: text/html";
        }

        headers ~= c.getCookieHeaders();

        return string.join(headers, "\n");
    }

    /* returns the current buffer */
    char[] getBuffer()
    {
        return buffer;
    }

    /* prints the headers and buffer */
    void flush()
    {
        char[] result;
        result = getHeaders()~"\n\n";
        result ~= buffer;
        printf(result);
    }
}

