module merten.cgi.url;

private import string;


/**
 * joins path and file.
 * urlJoin("/path/to/file.html", "../file.html") => "/path/file.html"
 */
char[] urlJoin(char[] path, char[] file)
{
    char[] result;

    if(file[0] == '/')
        return file;

    char[] getDirName(char[] path)
    {
        int lastSlash = string.rfind(path, '/');

        if(lastSlash > -1)
            path = path[0..lastSlash];
        else
            path = "";

        return path;
    }

    // while(file[0..3] == "../")
    for(int i = 0; i < string.count(file, "../"); i++)
    {
        path = getDirName(path);
    }

    file = string.replace(file, "../", "");

    if(file[0..2] == "./")
    {
        result = getDirName(path)~"/"~file[2..file.length];
    }
    else
    {
        result = getDirName(path)~"/"~file;
    }

    return result;
}
unittest
{
    char[] path = "/localhost/pizza/i/min/mage.html";
    char[] file1 = "../hej.html";
    char[] file2 = "../../../hej.html";
    char[] file3 = "roef/hej.html";
    char[] file4 = "./hej.html";
    char[] file5 = "./roef/hej.html";

    assert(urlJoin(path, file1) == "/localhost/pizza/i/hej.html");
    assert(urlJoin(path, file2) == "/localhost/hej.html");
    assert(urlJoin(path, file3) == "/localhost/pizza/i/min/roef/hej.html");
    assert(urlJoin(path, file4) == "/localhost/pizza/i/min/hej.html");
    assert(urlJoin(path, file5) == "/localhost/pizza/i/min/roef/hej.html");
}


/* stolen from Burton Radons urllib! */
char[] urlDecode(char[] url)
{
    char[] result;
    int length;

    int hexvalue(char ch)
    {
        if (ch >= '0' && ch <= '9')
            return ch - '0';
        if (ch >= 'a' && ch <= 'f')
            return ch - 'a' + 10;
        if (ch >= 'A' && ch <= 'F')
            return ch - 'A' + 10;
        throw new Error("URL has an encoding value that is outside of the hexadecimal range.");
    }

    for (int c; c < url.length; c ++, length ++)
        if (url[c] == '%')
        {
            if (c + 3 > url.length)
                throw new Error("URL has an encoding marker ('%') that is not followed by two characters.");
            c += 2;
        }

    if (length == url.length)
        return url;

    result = new char[length];
    length = 0;

    for (int c; c < url.length; c ++, length ++)
        if (url[c] == '%')
        {
            result[length] = hexvalue(url[c + 1]) * 16 + hexvalue(url[c + 2]);
            c += 2;
        }
        else
            result[length] = url[c];

    return result;
}

/** backwards compatible */
char[] urldecode(char[] url)
{
    return urlDecode(url);
}

private import c.stdio;

/* stolen from Burton Radons urllib! */
char[] urlEncode(char[] url)
{
    bit outofrange(char ch)
    {
        return (ch == '%' || ch <= 0x20 || ch >= 0x7F || ch == '<' || ch == '>'
            || ch == '"' || ch == '#' || ch == '{' || ch == '}' || ch == '|'
            || ch == '\\' || ch == '^' || ch == '~' || ch == '[' || ch == ']'
            || ch == '`');
    }

    char[] result;
    int length;

    for (int c; c < url.length; c ++, length ++)
    {
        if (outofrange(url[c]))
            length += 2;
    }

    if (length == url.length)
        return url;
    result = new char[length];
    length = 0;
    for (int c; c < url.length; c ++, length ++)
    {
        if (outofrange(url[c]))
        {
            sprintf(&result[length], "%%%02X", cast (ubyte)url[c]);
            length += 2;
        }
        else
            result[length] = url[c];
    }

    return result;
}